from .Helper import *
import SuperSaaS


class ClientTest(SupersaasTest):

    def test_api(self):
        self.assertIsNotNone(self.client.appointments)
        self.assertIsNotNone(self.client.forms)
        self.assertIsNotNone(self.client.users)

    def test_request(self):
        for method in ['GET', 'POST', 'PUT', 'DELETE']:
            self.assertIsNotNone(self.client.request(method, '/test', {'test': True}))

    def test_instance_configuration(self):
        self.client.configure(account_name='account', password='password')
        self.assertEqual('account', SuperSaaS.Client.instance().account_name)
        self.assertEqual('password', SuperSaaS.Client.instance().password)



from .BaseApi import BaseApi
from ..Models.Appointment import Appointment
from ..Models.Slot import Slot


class Appointments(BaseApi):
    def agenda(self, schedule_id, user_id, from_time=None):
        path = "/agenda/{}".format(self._validate_id(schedule_id))
        query = {
            'user': self._validate_present(user_id),
            'from': self._validate_datetime(from_time) if from_time else None
        }
        res = self.client.get(path, query)
        return self.__map_slots_or_bookings(res)

    def agenda_slots(self, schedule_id, user_id, from_time=None):
        path = "/agenda/{}".format(self._validate_id(schedule_id))
        query = {
            'user': self._validate_present(user_id),
            'from': self._validate_datetime(from_time) if from_time else None,
            'slot': 'true'
        }
        res = self.client.get(path, query)
        return self.__map_slots_or_bookings(res, True)

    def available(self, schedule_id, from_time=None, length_minutes=None, resource=None, full=None, limit=None):
        path = "/free/{}".format(self._validate_id(schedule_id))
        query = {
            'length': self._validate_number(length_minutes) if length_minutes else None,
            'from': self._validate_datetime(from_time) if from_time else None,
            'resource': resource,
            'full': 'true' if full else None,
            'maxresults': self._validate_number(limit) if limit else None
        }
        res = self.client.get(path, query)
        return self.__map_slots_or_bookings(res)

    def list(self, schedule_id, form=None, start_time=None, limit=None):
        path = "/bookings"
        query = {
            'schedule_id': self._validate_id(schedule_id),
            'form': 'true' if form else None,
            'start': self._validate_datetime(start_time) if start_time else None,
            'limit': self._validate_number(limit) if limit else None
        }
        res = self.client.get(path, query)
        return self.__map_slots_or_bookings(res)

    def get(self, schedule_id, appointment_id=None):
        query = {schedule_id: self._validate_id(schedule_id)}
        path = "/bookings/{}".format(self._validate_id(appointment_id))
        res = self.client.get(path, query)
        return Appointment(res)

    def create(self, schedule_id, user_id, attributes, form=None, webhook=None):
        path = "/bookings"
        params = {
            'schedule_id': schedule_id,
            'webhook': webhook,
            'user_id': self._validate_id(user_id),
            'form': form,
            'booking': {
                'start': attributes.get('start', ''),
                'finish': attributes.get('finish', ''),
                'name': self._validate_present(attributes.get('name', '')),
                'email': attributes.get('email', ''),
                'full_name': attributes.get('full_name', ''),
                'address': attributes.get('address', ''),
                'mobile': attributes.get('mobile', ''),
                'phone': attributes.get('phone', ''),
                'country': attributes.get('country', ''),
                'field_1': attributes.get('field_1', ''),
                'field_2': attributes.get('field_2', ''),
                'field_1_r': attributes.get('field_1_r', ''),
                'field_2_r': attributes.get('field_2_r', ''),
                'super_field': attributes.get('super_field', ''),
                'resource_id': attributes.get('resource_id', ''),
                'slot_id': attributes.get('slot_id', '')
            }
        }
        res = self.client.post(path, params)
        return Appointment(res)

    def update(self, schedule_id, appointment_id, attributes, webhook=None):
        path = "/bookings/{}".format(self._validate_id(appointment_id))
        params = {
            'schedule_id': schedule_id,
            'webhook': webhook,
            'form': attributes.get('form', ''),
            'booking': {
                'start': attributes.get('start', ''),
                'finish': attributes.get('finish', ''),
                'name': self._validate_present(attributes.get('name', '')),
                'email': attributes.get('email', ''),
                'full_name': attributes.get('full_name', ''),
                'address': attributes.get('address', ''),
                'mobile': attributes.get('mobile', ''),
                'phone': attributes.get('phone', ''),
                'country': attributes.get('country', ''),
                'field_1': attributes.get('field_1', ''),
                'field_2': attributes.get('field_2', ''),
                'field_1_r': attributes.get('field_1_r', ''),
                'field_2_r': attributes.get('field_2_r', ''),
                'super_field': attributes.get('super_field', ''),
                'resource_id': attributes.get('resource_id', ''),
                'slot_id': attributes.get('slot_id', '')
            }
        }
        res = self.client.put(path, params)
        return Appointment(res)

    def delete(self, schedule_id, appointment_id):
        path = "/bookings/{}".format(self._validate_id(appointment_id))
        query = {
            'schedule_id': schedule_id
        }
        return self.client.delete(path, None, query)

    def changes(self, schedule_id, from_time=None):
        path = "/changes/{}".format(self._validate_id(schedule_id))
        query = {
            'from': self._validate_datetime(from_time) if from_time else None
        }
        res = self.client.get(path, query)
        return self.__map_slots_or_bookings(res)

    def changes_slots(self, schedule_id, from_time=None):
        path = "/changes/{}".format(self._validate_id(schedule_id))
        query = {
            'from': self._validate_datetime(from_time) if from_time else None,
            'slot': 'true'
        }
        res = self.client.get(path, query)
        return self.__map_slots_or_bookings(res, True)

    def __map_slots_or_bookings(self, obj, slot=False):
        if slot:
            return [Slot(attributes) for attributes in obj.get('slots', [])]
        else:
            return [Appointment(attributes) for attributes in obj.get('bookings', [])]



import unittest
from SuperSaaS import Client
from SuperSaaS import Configuration


class SupersaasTest(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)

        self.__fileName = ""
        self.__file = None

        self.config = Configuration()
        self.client = Client(self.config)
        self.client.account_name = 'test'
        self.client.password = 'test'
        self.client.dry_run = True



from .BaseApi import BaseApi
from ..Models.User import User


class Users(BaseApi):
    def get(self, user_id=None):
        path = self.__user_path(user_id)
        res = self.client.get(path)
        return User(res)

    def list(self, form=None, limit=None, offset=None):
        path = self.__user_path()
        query = {
            'form': 'true' if form else None,
            'limit': self._validate_number(limit) if limit else None,
            'offset': self._validate_number(offset) if offset else None
        }
        res = self.client.get(path, query)
        return [User(attributes) for attributes in res]

    def create(self, attributes, user_id=None, webhook=None):
        path = self.__user_path(user_id)
        query = {'webhook': 'true' if webhook else None}
        params = {
            'user': {
                'name': self._validate_present(attributes.get('name','')),
                'email': attributes.get('email',''),
                'password': attributes.get('password',''),
                'full_name': attributes.get('full_name',''),
                'address': attributes.get('address',''),
                'mobile': attributes.get('mobile',''),
                'phone': attributes.get('phone',''),
                'country': attributes.get('country',''),
                'field_1': attributes.get('field_1',''),
                'field_2': attributes.get('field_2',''),
                'super_field': attributes.get('super_field',''),
                'credit': self._validate_number(attributes['credit']) if attributes.get('credit','') else None,
                'role': self._validate_options(attributes['role'], [3, 4, -1]) if attributes.get('role','') else None
            }
        }
        res = self.client.post(path, params, query)
        return User(res)

    def update(self, user_id, attributes, webhook=None):
        path = self.__user_path(user_id)
        query = {'webhook': 'true' if webhook else None}
        params = {
            'webhook': attributes.get('webhook', ''),
            'user': {
                'name': self._validate_present(attributes.get('name','')),
                'email': attributes.get('email',''),
                'password': attributes.get('password',''),
                'full_name': attributes.get('full_name',''),
                'address': attributes.get('address',''),
                'mobile': attributes.get('mobile',''),
                'phone': attributes.get('phone',''),
                'country': attributes.get('country',''),
                'field_1': attributes.get('field_1',''),
                'field_2': attributes.get('field_2',''),
                'super_field': attributes.get('super_field',''),
                'credit': self._validate_number(attributes['credit']) if attributes.get('credit','') else None,
                'role': self._validate_options(attributes['role'], [3, 4, -1]) if attributes.get('role','') else None
            }
        }
        return self.client.put(path, params, query)

    def delete(self, user_id):
        path = self.__user_path(self._validate_id(user_id))
        return self.client.delete(path)

    def __user_path(self, user_id=None):
        return "/users/{}".format(user_id) if user_id else "/users"



from .BaseApi import BaseApi
from ..Models.Schedule import Schedule
from ..Models.Resource import Resource


class Schedules(BaseApi):

    def list(self):
        path = "/schedules"
        res = self.client.get(path)
        return [Schedule(attributes) for attributes in res]

    def resources(self, schedule_id):
        query = {
            'schedule_id': self._validate_id(schedule_id)
        }
        path = "/resources"
        res = self.client.get(path, query)
        return [Resource(attributes) for attributes in res]



__all__ = [
    "BaseApi", "Appointments", "Forms", "Users", "Schedules"
]


from .BaseApi import BaseApi
from .Appointments import Appointments
from .Forms import Forms
from .Users import Users
from .Schedules import Schedules



from .Helper import *


class UsersTest(SupersaasTest):
    def test_get(self):
        user_id = 12345
        self.assertIsNotNone(self.client.users.get(user_id))

    def test_get_fk(self):
        user_fk = '6789fk'
        self.assertIsNotNone(self.client.users.get(user_fk))

    def test_list(self):
        self.assertIsNotNone(self.client.users.list(True, 10, 10))

    def test_create(self):
        self.assertIsNotNone(self.client.users.create(self.__user_attributes()))

    def test_create_fk(self):
        user_fk = '6789fk'
        self.assertIsNotNone(self.client.users.create(self.__user_attributes(), user_fk))

    def test_update(self):
        user_id = 12345
        self.assertIsNotNone(self.client.users.update(user_id, self.__user_attributes()))

    def test_delete(self):
        user_id = 12345
        self.assertIsNotNone(self.client.users.delete(user_id))

    def __user_attributes(self):
        return {
            'name': 'Test',
            'email': 'test@example.com',
            'password': 'pass123',
            'full_name': 'Tester Test',
            'address': '123 St, City',
            'mobile': '555-5555',
            'phone': '555-5555',
            'country': 'FR',
            'field_1': 'f 1',
            'field_2': 'f 2',
            'super_field': 'sf',
            'credit': 10,
            'role': 3,
            'webhook': 'true'
        }


import sys
import threading

try:
    from urllib.parse import urlparse, urlencode
    from urllib.request import urlopen, Request
    from urllib.error import HTTPError
except ImportError:
    from urlparse import urlparse
    from urllib import urlencode
    from urllib2 import urlopen, Request, HTTPError

try:
    import json
except ImportError:
    import simplejson as json

from base64 import b64encode

from SuperSaaS import API
from .Error import Error

PYTHON_VERSION = '.'.join([str(info) for info in sys.version_info])

API_VERSION = '1'
VERSION = '0.9.0'


class Client(object):
    __singleton_lock = threading.Lock()
    __singleton_instance = None

    @classmethod
    def instance(cls):
        if not cls.__singleton_instance:
            with cls.__singleton_lock:
                if not cls.__singleton_instance:
                    cls.__singleton_instance = cls(Configuration())
        return cls.__singleton_instance

    @classmethod
    def configure(cls, account_name, password, dry_run=False, verbose=False, host=None):
        cls.instance().account_name = account_name
        cls.instance().password = password
        cls.instance().dry_run = dry_run
        cls.instance().verbose = verbose
        cls.instance().host = host or cls.instance().host

    @classmethod
    def _user_agent(cls):
        return "SSS/{} Python/{} API/{}".format(VERSION, PYTHON_VERSION, API_VERSION)

    def __init__(self, configuration):
        self.account_name = configuration.account_name
        self.password = configuration.password
        self.host = configuration.host
        self.dry_run = configuration.dry_run
        self.verbose = configuration.verbose

        self.appointments = API.Appointments(self)
        self.forms = API.Forms(self)
        self.schedules = API.Schedules(self)
        self.users = API.Users(self)

        self.last_request = None

    def get(self, path, query=None):
        return self.request('GET', path, None, query)

    def post(self, path, params=None, query=None):
        return self.request('POST', path, params, query)

    def put(self, path, params=None, query=None):
        return self.request('PUT', path, params, query)

    def delete(self, path, params=None, query=None):
        return self.request('DELETE', path, params, query)

    def request(self, http_method, path, params=None, query=None):
        if params is None:
            params = {}
        if query is None:
            query = {}

        if not self.account_name:
            raise Error("Account name not configured. Call `SuperSaaS.Client.configure`.")
        if not self.password:
            raise Error("Account password not configured. Call `SuperSaaS.Client.configure`.")

        auth = b64encode('{}:{}'.format(self.account_name, self.password))
        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'User-Agent': self._user_agent(),
            'Authorization': "Basic {}".format(auth)
        }

        url = "{}/api{}.json".format(self.host, path)
        if http_method == 'GET':
            query = params
            params = None
        if query:
            querystring = urlencode(query)
            url = "{}?{}".format(url, querystring)

        if params:
            data = dict(filter(lambda item: item[1] is not None, params.items()))
            data = json.dumps(data)
        else:
            data = None
        req = Request(url, data, headers)

        if http_method == 'GET':
            req.get_method = lambda: http_method
        elif http_method == 'POST':
            req.get_method = lambda: http_method
        elif http_method == 'PUT':
            req.get_method = lambda: http_method
        elif http_method == 'DELETE':
            req.get_method = lambda: http_method
        else:
            raise Error("Invalid HTTP Method: {}. Only `GET`, `POST`, `PUT`, `DELETE` supported.".format(http_method))

        if self.verbose:
            print('')
            print("### SuperSaaS Client Request:")
            print('')
            print("#{} #{}".format(http_method, url))
            print('')
            print(data)
            print('')
            print("------------------------------")

        self.last_request = req
        if self.dry_run:
            return {}

        try:
            res = urlopen(req)
            data = json.load(res.read())

            if self.verbose:
                print('')
                print("Response:")
                print('')
                print(data)
                print('')
                print("==============================")

            return data
        except HTTPError, e:
            raise Error("HTTP Request Error ({}): {}".format(url, e.reason))


class Configuration(object):
    DEFAULT_HOST = 'https://www.supersaas.com'

    def __init__(self):
        self.account_name = ''
        self.password = ''
        self.host = self.DEFAULT_HOST
        self.dry_run = False
        self.verbose = False



from datetime import datetime
from .Helper import *


class AppointmentsTest(SupersaasTest):

    def test_get(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.get(schedule_id))

    def test_list(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.list(schedule_id, True, datetime.now(), 10))

    def test_create(self):
        user_id = 67890
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.create(schedule_id, user_id, self.__appointment_attributes(), True, True))

    def test_update(self):
        appointment_id = 67890
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.update(schedule_id, appointment_id, self.__appointment_attributes()))

    def test_agenda(self):
        schedule_id = 12345
        user_id = 67890
        self.assertIsNotNone(self.client.appointments.agenda(schedule_id, user_id))

    def test_agenda_slots(self):
        schedule_id = 12345
        user_id = 67890
        self.assertIsNotNone(self.client.appointments.agenda_slots(schedule_id, user_id))

    def test_available(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.available(schedule_id, datetime.now(), 10))

    def test_available_full(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.available(schedule_id, "2017-01-31 14:30:00", 10, 'My resource', False, 10))

    def test_changes(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.changes(schedule_id, "2017-01-31 14:30:00"))

    def test_changes_slots(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.appointments.changes_slots(schedule_id, datetime.now()))

    def test_delete(self):
        schedule_id = 12345
        appointment_id = 67890
        self.assertIsNotNone(self.client.appointments.delete(schedule_id, appointment_id))

    def __appointment_attributes(self):
        return {
            'description': 'Testing.',
            'name': 'Test',
            'email': 'test@example.com',
            'password': 'pass123',
            'full_name': 'Tester Test',
            'address': '123 St, City',
            'mobile': '555-5555',
            'phone': '555-5555',
            'country': 'FR',
            'field_1': 'f 1',
            'field_2': 'f 2',
            'field_1_r': 'f 1 r',
            'field_2_r': 'f 2 r',
            'super_field': 'sf'
        }



__all__ = [
    "BaseModel", "Form", "Slot", "Appointment", "User", "Resource", "Schedule"
]

from .BaseModel import BaseModel
from .Form import Form
from .Slot import Slot
from .Appointment import Appointment
from .User import User
from .Schedule import Schedule
from .Resource import Resource



from datetime import datetime
from .Helper import *


class FormsTest(SupersaasTest):
    def test_get(self):
        super_form_id = 12345
        self.assertIsNotNone(self.client.forms.get(super_form_id))

    def test_list(self):
        form_id = 67890
        self.assertIsNotNone(self.client.forms.list(form_id, datetime.now()))



from .BaseApi import BaseApi
from ..Models.Form import Form


class Forms(BaseApi):
    def get(self, form_id):
        path = "/forms"
        query = {
            'id': self._validate_id(form_id)
        }
        res = self.client.get(path, query)
        return Form(res)

    def list(self, form_id, from_time=None):
        path = "/forms"
        query = {
            'form_id': self._validate_id(form_id)
        }
        if from_time:
            query['from_time'] = self._validate_datetime(from_time)
        res = self.client.get(path, query)
        return [Form(attributes) for attributes in res]



from setuptools import setup, find_packages

setup(
    name='supersaas-api-client',
    version='0.10.1',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    description='Online bookings/appointments/calendars using the SuperSaaS scheduling platform  - https://supersaas.com',
    long_description='The SuperSaaS API provides services that can be used to add online booking and scheduling functionality to an existing website or CRM software.',
    author='Travis Dunn',
    author_email='travis@supersaas.com',
    keywords=['online appointment schedule', 'booking calendar', 'appointment book', 'reservation system', 
              'scheduling software', 'online booking system', 'scheduling system'],
    url='https://www.supersaas.com',
    install_requires=[
    ],
    project_urls={
        'Documentation': 'https://www.supersaas.com/info/dev',
        'Source': 'https://github.com/SuperSaaS/supersaas-python-api-client'
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Office/Business :: Scheduling'
    ],
)



import re
import datetime
from SuperSaaS.Error import Error


class BaseApi(object):
    INTEGER_REGEX = re.compile(r'\A[0-9]+\Z')
    DATETIME_REGEX = re.compile(r'\A\d{4}-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2}\Z')

    def __init__(self, client):
        self.client = client

    def _validate_id(self, value):
        return value

    def _validate_number(self, value):
        return self._validate_id(value)

    def _validate_present(self, value):
        if not value:
            raise Error("Required parameter is missing.")
        else:
            return value

    def _validate_datetime(self, value):
        if isinstance(value, datetime.datetime):
            return value.strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(value, str) and self.DATETIME_REGEX.match(value):
            return value
        else:
            raise Error("Invalid datetime parameter: {}. Provide a Time object or formatted 'YYYY-DD-MM%HH:MM:SS' string.".format(value))

    def _validate_options(self, value, options):
        if value in options:
            return value
        else:
            raise Error("Invalid option parameter: {}. Must be one of {}.".format(value, ', '.join(options)))




from datetime import datetime
from .Helper import *


class SchedulesTest(SupersaasTest):
    def test_list(self):
        self.assertIsNotNone(self.client.schedules.list())

    def test_resources(self):
        schedule_id = 12345
        self.assertIsNotNone(self.client.schedules.resources(schedule_id))